#ifndef IMAPFILTER_H
#define IMAPFILTER_H

#include <stdio.h>

#ifndef DATA_H
#include "data.h"
#endif

/* Error codes returned by functions. */
#define ERROR_SIGNAL			1
#define ERROR_TRIVIAL			2
#define ERROR_FILE_OPEN			3
#define ERROR_CONFIG_PARSE		4
#define ERROR_MEMORY_ALLOCATION		5
#define ERROR_NETWORK			6
#define ERROR_SSL			7
#define ERROR_UNDEFINED			8

/* Flags that control the program's options. */
#define OPTION_DETAILS_QUIET		0x01
#define OPTION_DETAILS_NORMAL		0x02
#define OPTION_DETAILS_VERBOSE		0x04
#define OPTION_DETAILS_CLEAR		0xf8
#define OPTION_HEADERS			0x08
#define OPTION_NAMESPACE		0x10

/* Capabilities of mail server. */
#define CAPABILITY_NONE			0x00
#define CAPABILITY_NAMESPACE		0x01

/* Flags for logger. */
#define LOG_WRITE			0
#define LOG_SERVER			1
#define LOG_USERNAME			2
#define LOG_MAILBOX			3
#define LOG_FILTER			4
#define LOG_ACTION			5
#define LOG_DESTINATION_MAILBOX		6

/* Buffer size of the IMAP command. */
#define SMALL_CMD			64
#define MEDIUM_CMD			256
#define BIG_CMD				8192

/* Buffer size of server's response. */
#define RESPONSE_BUF			(4096 + 1)
#define RESULT_BUF			512

/* Response codes. */
#define RESPONSE_BAD			-1
#define RESPONSE_OK			0
#define RESPONSE_NO			1
#define RESPONSE_BYE			2
#define RESPONSE_PREAUTH		3
#define RESPONSE_READONLY		4
#define RESPONSE_TRYCREATE		5

/* Buffer size of message's headers. */
#define HEADERS_BUF			16384

/* Length of mailbox namespace prefix. */
#define NAMESPACE_PREFIX_LEN		64


#define min(A, B)			((A) < (B) ? (A) : (B))
#define plural(A)			((A) == 1 ? "" : "s")


/* Namespace of mailboxes residing on mail server. */
struct namespace_t {
    char prefix[NAMESPACE_PREFIX_LEN];
    char delim;
};


/*	file.c		*/
int read_config(char *cfg);
#ifdef CHECK_PERMISSIONS
int check_permissions(char *cfg);
#endif
int parse_config(FILE * fp);
void set_options(char *line, regmatch_t * match);

/*	imapfilter.c	*/
void usage(void);

/*	imap.c		*/
unsigned int send_command(char *cmd);
#ifdef DEBUG
int imap_noop(void);
#endif
int imap_capability(void);
int imap_namespace(void);
int imap_logout(void);
int imap_login(char *user, char *pass);
/* int imap_examine(char *mbox); */
int imap_select(char *mbox);
int imap_status(char *mbox, char *items);
int imap_create(char *mbox);
int imap_search(char *search);
int imap_fetch(char *mesg, char *headers, int peek);
int imap_store(char *mesg, char *flags);
int imap_copy(char *mesg, char *mbox);
int imap_close(void);
/* int imap_expunge(void); */

/*	log.c		*/
void info(const char *info,...);
void verbose(const char *info,...);
void error(const char *errmsg,...);
void fatal(unsigned int errnum, const char *fatal,...);
void catch_signals(void);
void signal_handler(int sig);
int open_logfile(void);
int create_logfile(void);
int close_logfile(void);
void log_info(int flag, void *ptr);
char *get_time(void);

/*	memory.c	*/
void *xmalloc(size_t size);
void *xrealloc(void *ptr, size_t size);
char *xstrdup(const char *s);

/*	misc.c		*/
char *strcasestr(const char *haystack, const char *needle);
char *ultostr(unsigned long int num, int base);
char *xstrncpy(char *dest, const char *src, size_t size);

/*	request.c	*/
int test(void);
int check_capabilities(void);
int check_namespace(void);
int login(char *user, char *pass);
int select_mailbox(char *mbox);
int mailbox_status(char *mbox);
int close_mailbox(void);
int logout(void);

int apply_filters(filter_t ** filters);
int match_filter(filter_t * filter, char **mesgs);

void empty_fifo(mask_t ** mfifo);
void queue_fifo(mask_t ** mfifo, mask_t * mask);
mask_t *dequeue_fifo(mask_t ** mfifo);

char *generate_filter_and(mask_t * mask, unsigned int masknum, unsigned int masklen);
char *generate_filter_or(mask_t * mask, unsigned int masknum, unsigned int masklen);

int apply_action(char *mesgs, unsigned int *type, char *destmbox, char *args);
int action_delete(char *mesgs, char *args);
int action_copy(char *mesgs, char *destmbox, char *args);
int action_move(char *mesgs, char *destmbox, char *args);
int action_list(char *mesgs, char *args);

unsigned int convert_messages(char *mesgs);

/* 	response.c	*/
void receive_response(char *buf);
int server_response(unsigned int tag);
int greeting_response(void);
int capability_response(unsigned int tag);
int namespace_response(unsigned int tag, struct namespace_t * namesp);
int status_response(unsigned int tag, char *mbox);
int select_response(unsigned int tag);
int search_response(unsigned int tag, char **mesgs);
int fetch_response(unsigned int tag);
int copy_response(unsigned int tag);
int analyze_response(char *buf);

/*	socket.c	*/
#ifndef SSL_TLS
int init_connection(char *serv, unsigned short int port);
#else
int init_connection(char *serv, unsigned short int port,
		     unsigned int protocol);
#endif
int ssl_init(unsigned int protocol);
int close_connection(void);
int socket_read(char *buf);
int socket_write(char *data);

#endif				/* IMAPFILTER_H */
